/*
 * Project name:
     PS/2 Click Example (Demonstration on using PS/2 keyboard library)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140529 :
       - initial release
 * Description:
     In this example, key(s) pressed on the PS/2 keyboard are read and transferred
     to PC through serial port connection. Various basic keyboard activities are
     tested: "normal" keys, keys with <Shift> pressed, keys with <Caps Lock>
     pressed, numerical keypad ON/OFF and keys. The result is visible on PC, on
     USART Terminal tool.
 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev. Board:      EasyAVR7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     Ext. Modules:    PS/2 Click on mikroBUS1
                      http://www.mikroe.com/click/ps2/
                      Standard PS2 keyboard
                      http://en.wikipedia.org/wiki/Keyboard_(computing)
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Some keyboards with various multimedia attachments on them
       tend to "choke" the communication by constantly sending
       requests on various multimedia objects' status (volume, mouse pos. etc.).
       This may slow down the communication pace with the MCU.
     - Place PS/2 Click in mikroBUS socket 1.
     - Set J3 and J4 jumpers on EasyAVR v7 board in upper position for RS232
         or lower position for USB UART).
     - Set J5 jumper on 5V on EasyAVR v7 board.
     - Turn on SW10.1 and SW10.2 on EasyAVR v7 board.
 */

unsigned short keydata = 0, special = 0, down = 0;
 
sbit PS2_Data            at PINA6_bit;
sbit PS2_Clock_Input     at PINA5_bit;
sbit PS2_Clock_Output    at PORTA5_bit;
sbit PS2_Data_Direction  at DDA6_bit;
sbit PS2_Clock_Direction at DDA5_bit;

void main() {
  UART1_Init(115200);                                 // Initialize UART module at 115200bps
  Ps2_Config();                                       // Init PS/2 Keyboard
  Delay_ms(100);                                      // Wait for keyboard to finish
  UART1_Write_Text("Ready");
  UART1_Write(10);                                    // Line Feed
  UART1_Write(13);                                    // Carriage return

  do {
    if (Ps2_Key_Read(&keydata, &special, &down)) {
      if (down && (keydata == 16)) {                  // Backspace
         UART1_Write(0x08);
      }
      else if (down && (keydata == 13)) {             // Enter
        UART1_Write('\r');                            // send carriage return to usart terminal
        //UART1_Write('\n');                          // uncomment this line if usart terminal also expects line feed
                                                      // for new line transition
      }
      else if (down && !special && keydata) {
        UART1_Write(keydata);                         // Send key to UART1
      }
    }
    Delay_ms(1);                                      // debounce
  } while (1);
}